﻿function Set-SCVMMServerBitsPort
{
    param
    (
        [Parameter(Mandatory=$true, HelpMessage='VMM Server')]
        [String]
        $VMMServer,
        [Parameter(Mandatory=$false, HelpMessage='VMM Server')]
        [String]
        $Port='8500'
    )
    #VMM Management Server configuration
    Write-Host -ForegroundColor Yellow "BitsTcpPort on VMM Management Server $VMMServer is changed to $Port!"
    Invoke-Command -ComputerName $VMMServer -ScriptBlock{
        Set-ItemProperty -Path 'HKLM:\SOFTWARE\Microsoft\Microsoft System Center Virtual Machine Manager Server\Settings' -Name 'BITSTcpPort' -Value $Port -ErrorAction Continue
        $HTTPSRule=Get-NetFirewallRule -DisplayName 'HTTPS'
        $OldPort=$HTTPSRule|Get-NetFirewallPortFilter
        Set-NetFirewallPortFilter -InputObject $OldPort -LocalPort $Port -ErrorAction Continue
        $VMMBitsRule=Get-NetFirewallRule -DisplayName 'Virtual Machine Manager Server'
        foreach($Rule in $VMMBitsRule)
        {
            $OldPortVMM=$Rule|Get-NetFirewallPortFilter
            if($OldPortVMM.LocalPort -eq $OldPort.LocalPort)
            {
                Set-NetFirewallPortFilter -InputObject $OldPortVMM -LocalPort $Port -ErrorAction Continue
            }
        }
        Restart-Service -Name SCVMMService -WarningAction SilentlyContinue
        Restart-Service -Name SCVMMAgent -WarningAction SilentlyContinue
    } -ArgumentList $Port -ErrorAction Continue

    #Hyper-V hosts configuration
    $VMMManagedHosts=Get-SCVMHost -VMMServer $VMMServer
    foreach($VMMManagedHost in $VMMManagedHosts)
    {
        $Online=Test-NetConnection -ComputerName $VMMManagedHost
        if($Online.PingSucceeded -eq $true){
            Write-Host -ForegroundColor Green "Hyper-V host $VMMManagedHost is online!"
            Write-Host -ForegroundColor Yellow "$Port is added as port to firewall rule HTTPS on Hyper-V host $VMMManagedHost!"
            Invoke-Command -ComputerName $VMMManagedHost -ScriptBlock{
                param ($Port)
                $HTTPSRule=Get-NetFirewallRule -DisplayName 'HTTPS'
                $OldPort=$HTTPSRule|Get-NetFirewallPortFilter
                Set-NetFirewallPortFilter -InputObject $OldPort -LocalPort $Port -ErrorAction Continue
                Restart-Service -Name SCVMMAgent -WarningAction SilentlyContinue
            } -ArgumentList $Port -ErrorAction Continue
        }
        else{
           Write-Host -ForegroundColor Red "Hyper-V host $VMMManagedHost is offline!" 
        }
    }
}